@@ -73,7 +73,7 @@ from debusine.client.tests import server
 from debusine.client.tests.server import DebusineAioHTTPTestCase
 from debusine.tasks.models import LookupMultiple, OutputData
 from debusine.test import TestCase
-from debusine.test.utils import (
+from debusine.test.test_utils import (
     create_artifact_response,
     create_listing_response,
     create_work_request_response,
