@@ -850,7 +850,7 @@ class WorkspaceTests(TestCase):
         user = self.playground.get_default_user()
         self.assertQuerySetEqual(self.a.get_roles(user), [])
 
-        group.users.add(user)
+        self.playground.add_user(group, user)
         self.assertEqual(list(self.a.get_roles(user)), [Workspace.Roles.OWNER])
 
     def test_get_roles_multiple_groups(self) -> None:
@@ -865,10 +865,10 @@ class WorkspaceTests(TestCase):
         user = self.playground.get_default_user()
         self.assertQuerySetEqual(self.a.get_roles(user), [])
 
-        group1.users.add(user)
+        self.playground.add_user(group1, user)
         self.assertEqual(list(self.a.get_roles(user)), [Workspace.Roles.OWNER])
 
-        group2.users.add(user)
+        self.playground.add_user(group2, user)
         self.assertEqual(list(self.a.get_roles(user)), [Workspace.Roles.OWNER])
 
     def test_set_current_without_scope(self) -> None:
