@@ -47,13 +47,17 @@ from django.db.models.functions import Coalesce
 from django.urls import reverse
 from django.utils import timezone
 
-from debusine.artifacts.models import BareDataCategory, CollectionCategory
+from debusine.artifacts.models import (
+    BareDataCategory,
+    CollectionCategory,
+    DebusineHistoricalTaskRun,
+)
 from debusine.client.models import (
     LookupChildType,
     model_to_json_serializable_dict,
 )
 from debusine.db.context import context
-from debusine.db.models import CollectionItem
+from debusine.db.models import Collection, CollectionItem
 from debusine.db.models.permissions import (
     PartialCheckResult,
     PermissionUser,
@@ -62,10 +66,12 @@ from debusine.db.models.permissions import (
     permission_filter,
 )
 from debusine.db.models.scopes import Scope
+from debusine.db.models.worker_pools import WorkerPoolTaskExecutionStatistics
 from debusine.db.models.workspaces import WORKSPACE_ROLES, Workspace
 from debusine.server import notifications
 from debusine.tasks import BaseTask, TaskConfigError
 from debusine.tasks.models import (
+    ActionRecordInTaskHistory,
     ActionRetryWithDelays,
     ActionTypes,
     ActionUpdateCollectionWithArtifacts,
@@ -82,7 +88,6 @@ if TYPE_CHECKING:
     from django_stubs_ext.db.models import TypedModelMeta
 
     from debusine.db.models.auth import User
-    from debusine.db.models.collections import Collection
     from debusine.db.models.workers import Worker
     from debusine.server.workflows.models import (
         WorkRequestManualUnblockAction,
@@ -337,7 +342,6 @@ class WorkRequestManager(models.Manager["WorkRequest"]):
     ) -> "WorkRequest":
         """Create a workflow from a template and user-provided data."""
         # Import here to prevent circular imports
-        from debusine.db.models import Collection
         from debusine.server.workflows import Workflow
         from debusine.server.workflows.models import WorkRequestWorkflowData
 
@@ -838,7 +842,9 @@ class WorkRequest(models.Model):
     @output_data.setter
     def output_data(self, value: OutputData | None) -> None:
         """Set output_data_json from a pydantic structure."""
-        self.output_data_json = None if value is None else value.dict()
+        self.output_data_json = (
+            None if value is None else value.dict(exclude_unset=True)
+        )
 
     @property
     def workflow_data(self) -> "WorkRequestWorkflowData":
@@ -1329,6 +1335,20 @@ class WorkRequest(models.Model):
         self.output_data = output_data
         self.save()
 
+        if (
+            output_data is not None
+            and output_data.runtime_statistics is not None
+            and output_data.runtime_statistics.duration is not None
+            and self.worker is not None
+            and self.worker.worker_pool is not None
+        ):
+            WorkerPoolTaskExecutionStatistics.objects.create(
+                worker_pool=self.worker.worker_pool,
+                worker=self.worker,
+                scope=self.workspace.scope,
+                runtime=output_data.runtime_statistics.duration,
+            )
+
         if self.parent is not None:
             self._workflow_propagate_last_activity(
                 self.parent, self.completed_at
@@ -1368,6 +1388,9 @@ class WorkRequest(models.Model):
             actions[ActionTypes.UPDATE_COLLECTION_WITH_DATA]
         )
         self.process_retry_with_delays(actions[ActionTypes.RETRY_WITH_DELAYS])
+        self.process_record_in_task_history(
+            actions[ActionTypes.RECORD_IN_TASK_HISTORY]
+        )
 
     def get_triggered_actions(
         self,
@@ -1401,7 +1424,6 @@ class WorkRequest(models.Model):
     ) -> None:
         """Update collection with artifacts following event_reactions."""
         # local import to avoid circular dependency
-        from debusine.db.models.collections import CollectionItem
         from debusine.server.collections import (
             CollectionManagerInterface,
             ItemAdditionError,
@@ -1480,7 +1502,6 @@ class WorkRequest(models.Model):
     ) -> None:
         """Update collection with bare data following event_reactions."""
         # local import to avoid circular dependency
-        from debusine.db.models.collections import CollectionItem
         from debusine.server.collections import (
             CollectionManagerInterface,
             ItemAdditionError,
@@ -1574,6 +1595,55 @@ class WorkRequest(models.Model):
             wr_retried = self.retry()
             wr_retried.add_dependency(wr_delay)
 
+    def process_record_in_task_history(
+        self, actions: list[EventReaction]
+    ) -> None:
+        """Record a task run in a task-history collection."""
+        if (
+            actions
+            and self.started_at is not None
+            and self.dynamic_task_data is not None
+            and self.output_data is not None
+            and self.output_data.runtime_statistics is not None
+        ):
+            try:
+                collection = self.workspace.get_singleton_collection(
+                    user=self.created_by,
+                    category=CollectionCategory.TASK_HISTORY,
+                )
+            except Collection.DoesNotExist:
+                # No task-history collection; skip these actions.
+                return
+
+            default_subject: str | None = None
+            default_context: str | None = None
+            try:
+                task = self.get_task()
+            except Exception:
+                pass
+            else:
+                assert task.dynamic_data is not None
+                default_subject = task.dynamic_data.subject
+                default_context = task.dynamic_data.runtime_context
+
+            for action in actions:
+                assert isinstance(action, ActionRecordInTaskHistory)
+                collection.manager.add_bare_data(
+                    BareDataCategory.HISTORICAL_TASK_RUN,
+                    user=self.created_by,
+                    workflow=self.parent,
+                    data=DebusineHistoricalTaskRun(
+                        task_type=self.task_type,
+                        task_name=self.task_name,
+                        subject=action.subject or default_subject,
+                        context=action.context or default_context,
+                        timestamp=int(self.started_at.timestamp()),
+                        work_request_id=self.id,
+                        result=self.result,
+                        runtime_statistics=self.output_data.runtime_statistics,
+                    ),
+                )
+
     def mark_pending(self) -> bool:
         """Worker is ready for execution."""
         if self.status != self.Statuses.BLOCKED:
@@ -1817,12 +1887,10 @@ class WorkRequest(models.Model):
         if not (task_configuration := task.data.task_configuration):
             # We have no configuration to apply: recompute dynamic_task_data
             # for this worker
-            if dynamic_task_data := task.compute_dynamic_data(
-                TaskDatabase(self)
-            ):
-                self.dynamic_task_data = model_to_json_serializable_dict(
-                    dynamic_task_data, exclude_unset=True
-                )
+            dynamic_task_data = task.compute_dynamic_data(TaskDatabase(self))
+            self.dynamic_task_data = model_to_json_serializable_dict(
+                dynamic_task_data, exclude_unset=True
+            )
             return
 
         # Compute dynamic task data once to extract subject and
@@ -1830,10 +1898,6 @@ class WorkRequest(models.Model):
         # TODO: these values can be computed in a separate methods if the rest
         # of compute_dynamic_data becomes too expensive to run twice
         early_dynamic_task_data = task.compute_dynamic_data(TaskDatabase(self))
-        # TODO: remove this check once compute_dynamic_data is changed to
-        # never return None (see !1639)
-        if early_dynamic_task_data is None:
-            return  # pragma: no cover
 
         # Look up the debusine:task-configuration collection
         lookup_result = lookup_single(
@@ -1866,15 +1930,13 @@ class WorkRequest(models.Model):
             exc.add_parent_message("Configured task data is invalid")
             raise
 
-        if dynamic_task_data := configured_task.compute_dynamic_data(
+        dynamic_task_data = configured_task.compute_dynamic_data(
             TaskDatabase(self)
-        ):
-            dynamic_task_data.task_configuration_id = config_collection.pk
-            self.dynamic_task_data = model_to_json_serializable_dict(
-                dynamic_task_data, exclude_unset=True
-            )
-        else:
-            self.dynamic_task_data = None
+        )
+        dynamic_task_data.task_configuration_id = config_collection.pk
+        self.dynamic_task_data = model_to_json_serializable_dict(
+            dynamic_task_data, exclude_unset=True
+        )
 
     def assign_worker(self, worker: "Worker") -> None:
         """Assign worker and save it."""
@@ -2327,9 +2389,6 @@ class DeleteUnused:
 
     def perform_deletions(self) -> None:
         """Delete objects found in self.work_requests and self.collections."""
-        # Import here to prevent import loop
-        from debusine.db.models.collections import Collection
-
         CollectionItem.objects.filter(
             parent_collection__in=self.collections
         ).delete()
