@@ -24,13 +24,22 @@ from debusine.artifacts.models import (
     DebianUpload,
     DebusineSigningInput,
 )
-from debusine.assets import AssetCategory, KeyPurpose, SigningKeyData
+from debusine.assets import (
+    AWSProviderAccountConfiguration,
+    AWSProviderAccountCredentials,
+    AWSProviderAccountData,
+    AssetCategory,
+    CloudProvidersType,
+    DummyProviderAccountData,
+    KeyPurpose,
+    SigningKeyData,
+)
 from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
     ArtifactRelation,
+    Asset,
     DEFAULT_FILE_STORE_NAME,
-    DEFAULT_WORKSPACE_NAME,
     FileInArtifact,
     FileStore,
     Scope,
@@ -39,6 +48,12 @@ from debusine.db.models import (
 )
 from debusine.db.models.scopes import ScopeRole
 from debusine.db.playground import scenarios
+from debusine.server.worker_pools.models import (
+    AWSEC2WorkerPoolSpecification,
+    DummyWorkerPoolSpecification,
+    HetznerCloudWorkerPoolSpecification,
+    WorkerPoolLimits,
+)
 from debusine.server.workflows.models import WorkRequestWorkflowData
 from debusine.tasks.models import (
     BackendType,
@@ -81,7 +96,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
 
         workspace = playground.get_default_workspace()
         self.assertEqual(workspace.scope.file_stores.get(), file_store)
-        self.assertEqual(workspace.name, DEFAULT_WORKSPACE_NAME)
+        self.assertEqual(workspace.name, settings.DEBUSINE_DEFAULT_WORKSPACE)
 
     def test_user_password(self) -> None:
         """Check that the default user password is set when requested."""
@@ -115,6 +130,100 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(playground.create_workspace(scope=testscope), ws2)
         self.assertNotEqual(ws1, ws2)
 
+    def test_create_worker_without_pool(self) -> None:
+        playground = Playground()
+        worker = playground.create_worker()
+        self.assertIsNone(worker.worker_pool)
+        self.assertIsNone(worker.instance_created_at)
+
+    def test_create_worker_with_pool(self) -> None:
+        playground = Playground()
+        worker_pool = playground.create_worker_pool()
+        worker = playground.create_worker(worker_pool=worker_pool)
+        self.assertEqual(worker.worker_pool, worker_pool)
+        self.assertIsNotNone(worker.instance_created_at)
+
+    def test_create_worker_pool_explicit(self) -> None:
+        playground = Playground()
+        provider_account = playground.create_cloud_provider_account_asset()
+        specifications = AWSEC2WorkerPoolSpecification(launch_templates=[])
+        limits = WorkerPoolLimits(max_active_instances=42)
+        pool = playground.create_worker_pool(
+            name="foo",
+            enabled=False,
+            architectures=["amd64"],
+            tags=["a"],
+            provider_account=provider_account,
+            specifications=specifications,
+            instance_wide=False,
+            ephemeral=True,
+            limits=limits,
+        )
+        self.assertEqual(pool.name, "foo")
+        self.assertFalse(pool.enabled)
+        self.assertEqual(pool.architectures, ["amd64"])
+        self.assertEqual(pool.tags, ["a"])
+        self.assertEqual(pool.provider_account, provider_account)
+        self.assertEqual(pool.specifications, specifications.dict())
+        self.assertFalse(pool.instance_wide)
+        self.assertTrue(pool.ephemeral)
+        self.assertEqual(pool.limits, limits.dict())
+
+    def test_create_worker_pool_defaults(self) -> None:
+        playground = Playground()
+        pool = playground.create_worker_pool()
+        self.assertEqual(pool.name, "test")
+        self.assertTrue(pool.enabled)
+        self.assertEqual(pool.architectures, ["amd64", "i386"])
+        self.assertEqual(pool.tags, [])
+        self.assertIsNotNone(pool.provider_account, Asset)
+        self.assertEqual(
+            pool.specifications,
+            DummyWorkerPoolSpecification().dict(),
+        )
+        self.assertTrue(pool.instance_wide)
+        self.assertFalse(pool.ephemeral)
+        self.assertEqual(pool.limits, WorkerPoolLimits().dict())
+
+    def test_create_worker_pool_aws(self) -> None:
+        playground = Playground()
+        pool = playground.create_worker_pool(
+            provider_account=playground.create_cloud_provider_account_asset(
+                cloud_provider=CloudProvidersType.AWS
+            ),
+        )
+        self.assertEqual(
+            pool.specifications,
+            AWSEC2WorkerPoolSpecification(launch_templates=[]).dict(),
+        )
+
+    def test_create_worker_pool_hetzner(self) -> None:
+        playground = Playground()
+        pool = playground.create_worker_pool(
+            provider_account=playground.create_cloud_provider_account_asset(
+                cloud_provider=CloudProvidersType.HETZNER
+            ),
+        )
+        self.assertEqual(
+            pool.specifications,
+            HetznerCloudWorkerPoolSpecification(
+                server_type="cx22",
+                image_name="debian-12",
+            ).dict(),
+        )
+
+    def test_create_worker_pool_unknown_provider(self) -> None:
+        playground = Playground()
+        provider_account = playground.create_cloud_provider_account_asset()
+        provider_account.data = {"provider_type": "unknown"}
+        with context.disable_permission_checks():
+            provider_account.save()
+
+        with self.assertRaisesRegex(
+            NotImplementedError, r"No support for unknown"
+        ):
+            playground.create_worker_pool(provider_account=provider_account)
+
     def test_create_file_in_backend(self) -> None:
         """create_file_in_backend() returns a File and writes contents to it."""
         playground = self.enterContext(Playground(memory_file_store=False))
@@ -408,7 +517,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         data_model = asset.data_model
         assert isinstance(data_model, SigningKeyData)
         self.assertEqual(data_model.purpose, KeyPurpose.OPENPGP)
-        self.assertTrue(len(data_model.fingerprint) > 8)
+        self.assertGreater(len(data_model.fingerprint), 8)
         self.assertTrue(
             data_model.public_key.startswith(
                 "-----BEGIN PGP PUBLIC KEY BLOCK-----"
@@ -439,6 +548,38 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(data_model.public_key, "PUBLIC KEY")
         self.assertEqual(data_model.description, "Custom Asset")
 
+    def test_create_cloud_provider_account_asset_defaults(self) -> None:
+        playground = Playground()
+        asset = playground.create_cloud_provider_account_asset()
+        self.assertEqual(
+            asset.data, DummyProviderAccountData(name="test").dict()
+        )
+
+    def test_create_cloud_provider_account_asset_custom_data(self) -> None:
+        playground = Playground()
+        data = DummyProviderAccountData(name="test", secret="custom-secret")
+        asset = playground.create_cloud_provider_account_asset(data=data)
+        self.assertEqual(asset.data, data.dict())
+
+    def test_create_cloud_provider_account_asset_aws(self) -> None:
+        playground = Playground()
+        asset = playground.create_cloud_provider_account_asset(
+            name="aws-test", cloud_provider=CloudProvidersType.AWS
+        )
+        self.assertEqual(
+            asset.data,
+            AWSProviderAccountData(
+                name="aws-test",
+                configuration=AWSProviderAccountConfiguration(
+                    region_name="test-region"
+                ),
+                credentials=AWSProviderAccountCredentials(
+                    access_key_id="access-key",
+                    secret_access_key="secret-key",
+                ),
+            ).dict(),
+        )
+
     def test_create_asset_usage_defaults(self) -> None:
         """Test create_asset_usage with default workspace."""
         playground = Playground()
@@ -962,6 +1103,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertTrue(token.enabled)
         self.assertIsNone(token.user)
         self.assertFalse(hasattr(token, "worker"))
+        self.assertFalse(hasattr(token, "activating_worker"))
 
     def test_create_bare_token_disabled(self) -> None:
         """create_bare_token for a disabled token with no user and worker."""
@@ -970,6 +1112,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertFalse(token.enabled)
         self.assertIsNone(token.user)
         self.assertFalse(hasattr(token, "worker"))
+        self.assertFalse(hasattr(token, "activating_worker"))
 
     def test_create_user_token(self) -> None:
         """create_user_token returns an enabled token with user."""
@@ -978,6 +1121,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertTrue(token.enabled)
         self.assertEqual(token.user, playground.get_default_user())
         self.assertFalse(hasattr(token, "worker"))
+        self.assertFalse(hasattr(token, "activating_worker"))
 
     def test_create_user_token_with_user(self) -> None:
         """Test create_user_token with a given user."""
@@ -987,6 +1131,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertTrue(token.enabled)
         self.assertEqual(token.user, user)
         self.assertFalse(hasattr(token, "worker"))
+        self.assertFalse(hasattr(token, "activating_worker"))
 
     def test_create_user_token_disabled(self) -> None:
         """create_user_token returns a disabled token with user."""
@@ -995,22 +1140,63 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertFalse(token.enabled)
         self.assertIsNotNone(token.user)
         self.assertFalse(hasattr(token, "worker"))
+        self.assertFalse(hasattr(token, "activating_worker"))
 
     def test_create_worker_token(self) -> None:
-        """create_user_token returns an enabled token with user."""
+        """create_worker_token returns an enabled token."""
         playground = Playground()
         token = playground.create_worker_token()
         self.assertTrue(token.enabled)
         self.assertIsNone(token.user)
         self.assertIsNotNone(token.worker)
+        self.assertFalse(hasattr(token, "activating_worker"))
+
+    def test_create_worker_token_with_worker(self) -> None:
+        """create_worker_token returns a token with a worker."""
+        playground = Playground()
+        worker = playground.create_worker()
+        token = playground.create_worker_token(worker=worker)
+        self.assertTrue(token.enabled)
+        self.assertIsNone(token.user)
+        self.assertEqual(token.worker, worker)
+        self.assertFalse(hasattr(token, "activating_worker"))
 
     def test_create_worker_token_disabled(self) -> None:
-        """create_user_token returns a disabled token with user."""
+        """create_worker_token returns a disabled token."""
         playground = Playground()
         token = playground.create_worker_token(enabled=False)
         self.assertFalse(token.enabled)
         self.assertIsNone(token.user)
         self.assertIsNotNone(token.worker)
+        self.assertFalse(hasattr(token, "activating_worker"))
+
+    def test_create_worker_activation_token(self) -> None:
+        """create_worker_activation_token returns an enabled token."""
+        playground = Playground()
+        token = playground.create_worker_activation_token()
+        self.assertTrue(token.enabled)
+        self.assertIsNone(token.user)
+        self.assertFalse(hasattr(token, "worker"))
+        self.assertIsNotNone(token.activating_worker)
+
+    def test_create_worker_activation_token_with_worker(self) -> None:
+        """create_worker_activation_token returns a token with a worker."""
+        playground = Playground()
+        worker = playground.create_worker()
+        token = playground.create_worker_activation_token(worker=worker)
+        self.assertTrue(token.enabled)
+        self.assertIsNone(token.user)
+        self.assertFalse(hasattr(token, "worker"))
+        self.assertEqual(token.activating_worker, worker)
+
+    def test_create_worker_activation_token_disabled(self) -> None:
+        """create_worker_activation_token returns a disabled token."""
+        playground = Playground()
+        token = playground.create_worker_activation_token(enabled=False)
+        self.assertFalse(token.enabled)
+        self.assertIsNone(token.user)
+        self.assertFalse(hasattr(token, "worker"))
+        self.assertIsNotNone(token.activating_worker)
 
     def test_create_group(self) -> None:
         playground = Playground()
