@@ -60,6 +60,10 @@ service_urlpatterns: list[URLPattern | URLResolver] = [
         "signon/",
         include("debusine.web.urls.signon", namespace="signon"),
     ),
+    path(
+        "enroll/",
+        include("debusine.web.urls.enroll", namespace="enroll"),
+    ),
 ]
 
 
