@@ -31,6 +31,7 @@ from debusine.server.collections.debian_suite import DebianSuiteManager
 from debusine.server.collections.debian_suite_lintian import (
     DebianSuiteLintianManager,
 )
+from debusine.server.collections.task_history import TaskHistoryManager
 from debusine.server.collections.workflow_internal import (
     WorkflowInternalManager,
 )
@@ -43,5 +44,6 @@ __all__ = [
     "DebianSuiteManager",
     "ItemAdditionError",
     "ItemRemovalError",
+    "TaskHistoryManager",
     "WorkflowInternalManager",
 ]
