@@ -77,6 +77,9 @@ class AutopkgtestWorkflow(
         """Create an autopkgtest work request for a single architecture."""
         assert self.work_request is not None
 
+        source_artifact = workflow_utils.locate_debian_source_package_lookup(
+            self, "source_artifact", self.data.source_artifact
+        )
         filtered_binary_artifacts = (
             workflow_utils.filter_artifact_lookup_by_arch(
                 self, self.data.binary_artifacts, (architecture, "all")
@@ -89,7 +92,7 @@ class AutopkgtestWorkflow(
         )
         task_data = AutopkgtestData(
             input=AutopkgtestInput(
-                source_artifact=self.data.source_artifact,
+                source_artifact=source_artifact,
                 binary_artifacts=filtered_binary_artifacts,
                 context_artifacts=filtered_context_artifacts,
             ),
@@ -115,7 +118,7 @@ class AutopkgtestWorkflow(
                 step=f"autopkgtest-{architecture}",
             ),
         )
-        self.requires_artifact(wr, self.data.source_artifact)
+        self.requires_artifact(wr, source_artifact)
         self.requires_artifact(wr, filtered_binary_artifacts)
         self.requires_artifact(wr, filtered_context_artifacts)
         self.provides_artifact(
