@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -9,11 +9,12 @@
 
 """Pydantic models used by debusine workflows."""
 
+import re
 from datetime import datetime
 from enum import StrEnum
 from typing import Any
 
-from debusine.artifacts.models import KeyPurpose
+from debusine.assets import KeyPurpose
 
 try:
     import pydantic.v1 as pydantic
@@ -28,12 +29,12 @@ from debusine.tasks.models import (
     AutopkgtestTimeout,
     BackendType,
     BaseTaskData,
+    ExtraRepository,
     LintianFailOnSeverity,
     LintianOutput,
     LookupMultiple,
     LookupSingle,
     SbuildBinNMU,
-    SbuildExtraRepository,
     SbuildInput,
     empty_lookup_multiple,
 )
@@ -126,7 +127,7 @@ class SbuildWorkflowData(BaseWorkflowData):
     )
     arch_all_host_architecture: str = "amd64"
     environment_variant: str | None = None
-    extra_repositories: list[SbuildExtraRepository] | None = None
+    extra_repositories: list[ExtraRepository] | None = None
     binnmu: SbuildBinNMU | None = None
     build_profiles: list[str] | None = None
     retry_delays: list[str] | None = pydantic.Field(default=None, min_items=1)
@@ -182,7 +183,7 @@ class PackageUploadWorkflowData(BaseWorkflowData):
     merge_uploads: bool = False
     since_version: str | None = None
     target_distribution: str | None = None
-    key: LookupSingle | None = None
+    key: str | None = None
     require_signature: bool = True
     target: PackageUploadTarget
     delayed_days: int | None = None
@@ -202,7 +203,7 @@ class MakeSignedSourceWorkflowData(BaseWorkflowData):
     codename: str
     architectures: list[str]
     purpose: KeyPurpose
-    key: LookupSingle
+    key: str
     sbuild_backend: BackendType = BackendType.AUTO
 
 
@@ -216,6 +217,7 @@ class PiupartsWorkflowData(BaseWorkflowData):
     architectures: list[str] | None = None
     backend: BackendType = BackendType.AUTO
     arch_all_host_architecture: str = "amd64"
+    extra_repositories: list[ExtraRepository] | None = None
 
 
 class AutopkgtestWorkflowData(BaseWorkflowData):
@@ -234,6 +236,7 @@ class AutopkgtestWorkflowData(BaseWorkflowData):
     backend: BackendType = BackendType.AUTO
     architectures: list[str] = pydantic.Field(default_factory=list)
     arch_all_host_architecture: str = "amd64"
+    extra_repositories: list[ExtraRepository] | None = None
 
     include_tests: list[str] = pydantic.Field(default_factory=list)
     exclude_tests: list[str] = pydantic.Field(default_factory=list)
@@ -263,6 +266,7 @@ class ReverseDependenciesAutopkgtestWorkflowData(BaseWorkflowData):
     arch_all_host_architecture: str = "amd64"
     packages_allowlist: list[str] | None = None
     packages_denylist: list[str] = []
+    extra_repositories: list[ExtraRepository] | None = None
 
     debug_level: int = pydantic.Field(default=0, ge=0, le=3)
 
@@ -294,6 +298,7 @@ class QAWorkflowData(BaseWorkflowData):
     vendor: str
     codename: str
     architectures: list[str] | None = None
+    extra_repositories: list[ExtraRepository] | None = None
 
     architectures_allowlist: list[str] | None = None
     architectures_denylist: list[str] | None = None
@@ -342,6 +347,7 @@ class DebianPipelineWorkflowData(BaseWorkflowData):
     architectures_allowlist: list[str] | None = None
     architectures_denylist: list[str] | None = None
     arch_all_host_architecture: str = "amd64"
+    extra_repositories: list[ExtraRepository] | None = None
 
     signing_template_names: dict[str, list[str]] = {}
 
@@ -363,11 +369,11 @@ class DebianPipelineWorkflowData(BaseWorkflowData):
 
     enable_make_signed_source: bool = False
     make_signed_source_purpose: KeyPurpose | None = None
-    make_signed_source_key: LookupSingle | None = None
+    make_signed_source_key: str | None = None
 
     enable_upload: bool = False
 
-    upload_key: LookupSingle | None = None
+    upload_key: str | None = None
     upload_require_signature: bool = True
     upload_include_source: bool = True
     upload_include_binaries: bool = True
@@ -422,3 +428,27 @@ class PackagePublishWorkflowData(BaseWorkflowData):
                 '"source_artifact" or "binary_artifacts" must be set'
             )
         return values
+
+
+class ExperimentWorkspaceData(BaseWorkflowData):
+    """``create_experiment_workspace`` workflow data."""
+
+    experiment_name: str
+    public: bool = True
+    owner_group: str | None = None
+    workflow_template_names: list[str] = []
+    expiration_delay: int | None = 60
+
+    @pydantic.validator("experiment_name")
+    @classmethod
+    def validate_experiment_name(cls, data: str) -> str:
+        """
+        Validate the experiment name.
+
+        Valid characters are the sames as workspace/group names, minus the '-'
+        """
+        # In pydantic 1.10 this could be done with constr, but mypy currently
+        # complains about their syntax
+        if not re.match(r"^[A-Za-z][A-Za-z0-9+._]*$", data):
+            raise ValueError("experiment name contains invalid characters")
+        return data
