@@ -106,12 +106,14 @@ class PackageUploadWorkflow(
 
     def populate(self) -> None:
         """Create package upload work requests."""
+        workflow_root = self.work_request.get_workflow_root()
+
         if (source_artifact := self.data.source_artifact) is not None:
             artifact = lookup_single(
                 source_artifact,
                 self.workspace,
                 user=self.work_request.created_by,
-                workflow_root=self.work_request.get_workflow_root(),
+                workflow_root=workflow_root,
                 expect_type=LookupChildType.ARTIFACT,
             ).artifact
             since_version = self.data.since_version
@@ -137,12 +139,14 @@ class PackageUploadWorkflow(
             self.data.binary_artifacts,
             self.workspace,
             user=self.work_request.created_by,
-            workflow_root=self.work_request.get_workflow_root(),
+            workflow_root=workflow_root,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
         )
 
         for result in results:
-            self._artifacts_pending_upload.append(reconstruct_lookup(result))
+            self._artifacts_pending_upload.append(
+                reconstruct_lookup(result, workflow_root=workflow_root)
+            )
 
         self._populate_package_uploads()
 
