@@ -13,7 +13,7 @@ import json
 import re
 from collections.abc import Iterable
 from dataclasses import dataclass
-from functools import cached_property, partial
+from functools import partial
 from itertools import chain
 from typing import Any, Self
 
@@ -30,7 +30,6 @@ from django.db.models import (
 )
 from django.db.models.fields.json import KT
 
-from debusine.artifacts import SourcePackage
 from debusine.artifacts.models import (
     ArtifactCategory,
     DebianBinaryPackage,
@@ -39,8 +38,9 @@ from debusine.artifacts.models import (
     DebianUpload,
 )
 from debusine.client.models import LookupChildType
-from debusine.db.models import Artifact, CollectionItem, WorkRequest
+from debusine.db.models import CollectionItem, WorkRequest
 from debusine.server.collections.lookup import lookup_multiple, lookup_single
+from debusine.server.workflows import workflow_utils
 from debusine.server.workflows.base import Workflow, orchestrate_workflow
 from debusine.server.workflows.models import (
     AutopkgtestWorkflowData,
@@ -216,7 +216,7 @@ class ReverseDependenciesAutopkgtestWorkflow(
             workflow_root=self.work_request.get_workflow_root(),
             expect_type=LookupChildType.ARTIFACT,
         ).artifact
-        source_artifact = self.locate_debian_source_package(
+        source_artifact = workflow_utils.locate_debian_source_package(
             "source_artifact", source_artifact
         )
         source_artifact_data = source_artifact.create_data()
@@ -423,27 +423,15 @@ class ReverseDependenciesAutopkgtestWorkflow(
         for source, binaries in self.get_reverse_dependencies():
             self._populate_single(source, binaries)
 
-    @cached_property
-    def source_package(self) -> Artifact:
-        """Source package artifact."""
-        artifact = lookup_single(
-            self.data.source_artifact,
-            self.workspace,
-            user=self.work_request.created_by,
-            expect_type=LookupChildType.ARTIFACT,
-        ).artifact
-        return self.locate_debian_source_package("source_artifact", artifact)
-
-    @cached_property
-    def source_package_data(self) -> DebianSourcePackage:
-        """Parsed source package artifact data."""
-        return SourcePackage.create_data(self.source_package.data)
-
-    def compute_dynamic_data(
+    def build_dynamic_data(
         self, task_database: TaskDatabaseInterface  # noqa: U100
     ) -> BaseDynamicTaskData:
-        """Compute dynamic data for this workflow."""
-        source_data = self.source_package_data
+        """
+        Compute dynamic data for this workflow.
+
+        :subject: package name of ``source_artifact``
+        """
+        source_data = workflow_utils.source_package_data(self)
         return BaseDynamicTaskData(
             subject=source_data.name,
             parameter_summary=f"{source_data.name}_{source_data.version}",
