@@ -306,7 +306,6 @@ class AutopkgtestWorkflowTests(TestCase):
                     AutopkgtestWorkflow(autopkgtest).populate()
 
             root = self.playground.create_workflow(task_name="examplepipeline")
-            assert (internal_collection := root.internal_collection) is not None
 
             root.mark_running()
             orchestrate_workflow(root)
@@ -335,10 +334,182 @@ class AutopkgtestWorkflowTests(TestCase):
                             ],
                             "binary_artifacts": sorted(
                                 [
-                                    f"{internal_collection.id}@collections/"
+                                    f"internal@collections/"
                                     f"name:build-{architecture}",
-                                    f"{internal_collection.id}@collections/"
-                                    f"name:build-all",
+                                    "internal@collections/name:build-all",
+                                ]
+                            ),
+                            "context_artifacts": [],
+                        },
+                        "host_architecture": architecture,
+                        "environment": "debian/match:codename=sid",
+                        "extra_repositories": [
+                            {
+                                "url": "http://example.com/",
+                                "suite": "bookworm",
+                                "components": ["main"],
+                            }
+                        ],
+                        "backend": "unshare",
+                        "include_tests": [],
+                        "exclude_tests": [],
+                        "debug_level": 0,
+                        "extra_environment": {},
+                        "needs_internet": AutopkgtestNeedsInternet.RUN,
+                        "fail_on": {},
+                        "timeout": None,
+                    },
+                )
+                self.assertEqual(
+                    child.event_reactions_json,
+                    {
+                        "on_creation": [],
+                        "on_failure": [],
+                        "on_success": [
+                            {
+                                "action": "update-collection-with-artifacts",
+                                "collection": "internal@collections",
+                                "name_template": f"autopkgtest-{architecture}",
+                                "variables": None,
+                                "artifact_filters": {
+                                    "category": ArtifactCategory.AUTOPKGTEST
+                                },
+                            }
+                        ],
+                        "on_unblock": [],
+                    },
+                )
+                self.assertQuerySetEqual(
+                    child.dependencies.all(),
+                    list(
+                        WorkRequest.objects.filter(
+                            task_type=TaskTypes.WORKER,
+                            task_name="sbuild",
+                            task_data__host_architecture__in={
+                                architecture,
+                                "all",
+                            },
+                        )
+                    ),
+                )
+                self.assertEqual(
+                    child.workflow_data_json,
+                    {
+                        "display_name": f"autopkgtest {architecture}",
+                        "step": f"autopkgtest-{architecture}",
+                    },
+                )
+
+            # Population is idempotent.
+            AutopkgtestWorkflow(autopkgtest).populate()
+            children = list(WorkRequest.objects.filter(parent=autopkgtest))
+            self.assertEqual(len(children), len(architectures))
+
+    def test_populate_upload(self) -> None:
+        """The workflow accepts debian:upload source artifacts."""
+        architectures = ("amd64", "i386")
+        upload_artifacts = self.playground.create_upload_artifacts()
+
+        with preserve_task_registry():
+
+            class ExamplePipeline(
+                TestWorkflow[BaseWorkflowData, BaseDynamicTaskData]
+            ):
+                """Pipeline workflow that runs sbuild and autopkgtest."""
+
+                def populate(self) -> None:
+                    """Populate the pipeline."""
+                    sbuild = self.work_request.create_child(
+                        task_type=TaskTypes.WORKFLOW,
+                        task_name="sbuild",
+                        task_data=SbuildWorkflowData(
+                            input=SbuildInput(
+                                source_artifact=upload_artifacts.upload.id
+                            ),
+                            target_distribution="debian:sid",
+                            architectures=["all", *architectures],
+                        ),
+                    )
+                    for architecture in ("all", *architectures):
+                        child = sbuild.create_child(
+                            task_name="sbuild",
+                            task_data=SbuildData(
+                                input=SbuildInput(
+                                    source_artifact=(
+                                        f"{upload_artifacts.source.id}"
+                                        f"@artifacts"
+                                    )
+                                ),
+                                host_architecture=architecture,
+                                environment="debian/match:codename=sid",
+                            ),
+                        )
+                        self.provides_artifact(
+                            child,
+                            ArtifactCategory.UPLOAD,
+                            f"build-{architecture}",
+                            data={"architecture": architecture},
+                        )
+
+                    autopkgtest = self.work_request.create_child(
+                        task_type=TaskTypes.WORKFLOW,
+                        task_name="autopkgtest",
+                        task_data=AutopkgtestWorkflowData(
+                            source_artifact=upload_artifacts.upload.id,
+                            binary_artifacts=LookupMultiple.parse_obj(
+                                [
+                                    f"internal@collections/"
+                                    f"name:build-{architecture}"
+                                    for architecture in ("all", *architectures)
+                                ]
+                            ),
+                            vendor="debian",
+                            codename="sid",
+                            extra_repositories=[
+                                ExtraRepository.parse_obj(
+                                    {
+                                        "url": "http://example.com/",
+                                        "suite": "bookworm",
+                                        "components": ["main"],
+                                    }
+                                )
+                            ],
+                        ),
+                    )
+                    AutopkgtestWorkflow(autopkgtest).populate()
+
+            root = self.playground.create_workflow(task_name="examplepipeline")
+
+            root.mark_running()
+            orchestrate_workflow(root)
+
+            autopkgtest = WorkRequest.objects.get(
+                task_type=TaskTypes.WORKFLOW,
+                task_name="autopkgtest",
+                parent=root,
+            )
+            children = list(
+                WorkRequest.objects.filter(parent=autopkgtest).order_by(
+                    "task_data__host_architecture"
+                )
+            )
+            self.assertEqual(len(children), len(architectures))
+            for child, architecture in zip(children, architectures):
+                self.assertEqual(child.status, WorkRequest.Statuses.BLOCKED)
+                self.assertEqual(child.task_type, TaskTypes.WORKER)
+                self.assertEqual(child.task_name, "autopkgtest")
+                self.assertEqual(
+                    child.task_data,
+                    {
+                        "input": {
+                            "source_artifact": (
+                                f"{upload_artifacts.source.id}@artifacts"
+                            ),
+                            "binary_artifacts": sorted(
+                                [
+                                    f"internal@collections/"
+                                    f"name:build-{architecture}",
+                                    "internal@collections/name:build-all",
                                 ]
                             ),
                             "context_artifacts": [],
