@@ -12,289 +12,17 @@
 import logging
 from unittest import mock
 
-from debusine.artifacts.models import ArtifactCategory
-from debusine.db.models import ArtifactRelation, CollectionItem, WorkRequest
-from debusine.server.collections.lookup import LookupResult
+from debusine.db.models import WorkRequest
 from debusine.server.workflows.base import (
-    Workflow,
     WorkflowRunError,
     orchestrate_workflow,
 )
 from debusine.server.workflows.models import BaseWorkflowData
-from debusine.server.workflows.noop import NoopWorkflow
-from debusine.tasks import TaskConfigError
+from debusine.server.workflows.tests.helpers import TestWorkflow
 from debusine.tasks.models import BaseDynamicTaskData, TaskTypes
 from debusine.tasks.server import TaskDatabaseInterface
-from debusine.tasks.tests.helper_mixin import TestTaskMixin
 from debusine.test.django import TestCase
-from debusine.test.utils import preserve_task_registry
-
-
-class WorkflowClassTests(TestCase):
-    """Test the Workflow base class."""
-
-    def test_ensure_artifact_categories_matches(self) -> None:
-        """Test with a matching artifact."""
-        artifact, _ = self.playground.create_artifact(
-            category=ArtifactCategory.TEST
-        )
-        Workflow.ensure_artifact_categories(
-            configuration_key="test",
-            category=artifact.category,
-            expected=[ArtifactCategory.TEST, ArtifactCategory.UPLOAD],
-        )
-
-    def test_ensure_artifact_categories_mismatches(self) -> None:
-        """Test with a mis-matching artifact."""
-        artifact, _ = self.playground.create_artifact(
-            category=ArtifactCategory.TEST
-        )
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^test: unexpected artifact category: 'debusine:test'. "
-            r"Valid categories: \['debian:system-image', "
-            r"'debian:system-tarball'\]$",
-        ):
-            Workflow.ensure_artifact_categories(
-                configuration_key="test",
-                category=artifact.category,
-                expected=[
-                    ArtifactCategory.SYSTEM_TARBALL,
-                    ArtifactCategory.SYSTEM_IMAGE,
-                ],
-            )
-
-    def test_follow_artifact_relation_upload(self) -> None:
-        """Test finding a source package from an upload."""
-        upload_artifacts = self.playground.create_upload_artifacts()
-        test_artifact, _ = self.playground.create_artifact()
-
-        # Create some extra unrelated relations
-        self.playground.create_artifact_relation(
-            artifact=upload_artifacts.upload,
-            target=upload_artifacts.binaries[0],
-            relation_type=ArtifactRelation.Relations.BUILT_USING,
-        )
-        self.playground.create_artifact_relation(
-            artifact=upload_artifacts.upload,
-            target=test_artifact,
-            relation_type=ArtifactRelation.Relations.EXTENDS,
-        )
-
-        artifact = Workflow.follow_artifact_relation(
-            upload_artifacts.upload,
-            ArtifactRelation.Relations.EXTENDS,
-            ArtifactCategory.SOURCE_PACKAGE,
-        )
-        self.assertEqual(artifact, upload_artifacts.source)
-
-    def test_follow_artifact_relation_not_found(self) -> None:
-        """Test with a lone debian:upload (no relations)."""
-        upload_artifacts = self.playground.create_upload_artifacts(source=False)
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"Unable to find an artifact of category debian:source-package "
-            r'with a relationship of type extends from ".* debian:upload .*"',
-        ):
-            Workflow.follow_artifact_relation(
-                upload_artifacts.upload,
-                ArtifactRelation.Relations.EXTENDS,
-                ArtifactCategory.SOURCE_PACKAGE,
-            )
-
-    def test_follow_artifact_relation_multiple_found(self) -> None:
-        """Test with a debian:upload extending multiple srcpkgs."""
-        upload_artifacts = self.playground.create_upload_artifacts(source=True)
-        source2 = self.playground.create_source_artifact()
-        self.playground.create_artifact_relation(
-            artifact=upload_artifacts.upload,
-            target=source2,
-            relation_type=ArtifactRelation.Relations.EXTENDS,
-        )
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"Multiple artifacts of category debian:source-package with "
-            r'a relationship of type extends from ".* debian:upload .*" found',
-        ):
-            Workflow.follow_artifact_relation(
-                upload_artifacts.upload,
-                ArtifactRelation.Relations.EXTENDS,
-                ArtifactCategory.SOURCE_PACKAGE,
-            )
-
-    def test_locate_debian_source_package_with_srcpkg(self) -> None:
-        """Test with a debian:source-package."""
-        source_artifact = self.playground.create_source_artifact()
-        result = Workflow.locate_debian_source_package(
-            "source_artifact", source_artifact
-        )
-        self.assertEqual(result, source_artifact)
-
-    def test_locate_debian_source_package_with_upload(self) -> None:
-        """Test with a debian:source-package."""
-        upload_artifacts = self.playground.create_upload_artifacts()
-        result = Workflow.locate_debian_source_package(
-            "upload_artifact", upload_artifacts.upload
-        )
-        self.assertEqual(result, upload_artifacts.source)
-
-    def test_locate_debian_source_package_with_junk(self) -> None:
-        """Test with a debian:source-package."""
-        artifact, _ = self.playground.create_artifact(
-            category=ArtifactCategory.TEST
-        )
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^test_artifact: unexpected artifact category: "
-            r"'debusine:test'. Valid categories: "
-            r"\['debian:source-package', 'debian:upload'\]$",
-        ):
-            Workflow.locate_debian_source_package("test_artifact", artifact)
-
-    def test_get_source_package_names(self) -> None:
-        binary_artifact_1 = (
-            self.playground.create_minimal_binary_package_artifact(
-                srcpkg_name="hello"
-            )
-        )
-        binary_artifact_2 = (
-            self.playground.create_minimal_binary_package_artifact(
-                srcpkg_name="hello"
-            )
-        )
-        binaries_artifact = (
-            self.playground.create_minimal_binary_packages_artifact(
-                srcpkg_name="linux-base"
-            )
-        )
-        upload_artifact = self.playground.create_upload_artifacts(
-            src_name="firefox"
-        ).upload
-
-        lookup_result = [
-            LookupResult(
-                result_type=CollectionItem.Types.ARTIFACT,
-                artifact=binary_artifact_1,
-            ),
-            LookupResult(
-                result_type=CollectionItem.Types.ARTIFACT,
-                artifact=binary_artifact_2,
-            ),
-            LookupResult(
-                result_type=CollectionItem.Types.ARTIFACT,
-                artifact=binaries_artifact,
-            ),
-            LookupResult(
-                result_type=CollectionItem.Types.ARTIFACT,
-                artifact=upload_artifact,
-            ),
-            LookupResult(
-                result_type=CollectionItem.Types.BARE,
-                collection_item=CollectionItem(
-                    data={
-                        "promise_category": ArtifactCategory.BINARY_PACKAGE,
-                        "source_package_name": "python3",
-                    }
-                ),
-            ),
-        ]
-
-        self.assertEqual(
-            Workflow.get_source_package_names(
-                lookup_result,
-                configuration_key="testing",
-                artifact_expected_categories=(
-                    ArtifactCategory.BINARY_PACKAGE,
-                    ArtifactCategory.BINARY_PACKAGES,
-                    ArtifactCategory.UPLOAD,
-                ),
-            ),
-            ["firefox", "hello", "linux-base", "python3"],
-        )
-
-    def test_get_source_package_name_invalid_artifact_category(self) -> None:
-        source_artifact = self.playground.create_source_artifact(name="hello")
-
-        lookup_result = [
-            LookupResult(
-                result_type=CollectionItem.Types.ARTIFACT,
-                artifact=source_artifact,
-            )
-        ]
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^testing: unexpected artifact category: 'debian:source-package'. "
-            r"Valid categories: "
-            r"\['debian:binary-package', 'debian:binary-packages'\]$",
-        ):
-            Workflow.get_source_package_names(
-                lookup_result,
-                configuration_key="testing",
-                artifact_expected_categories=(
-                    ArtifactCategory.BINARY_PACKAGE,
-                    ArtifactCategory.BINARY_PACKAGES,
-                ),
-            )
-
-    def test_get_source_package_name_invalid_artifact_in_promise_category(
-        self,
-    ) -> None:
-        lookup_result = [
-            LookupResult(
-                result_type=CollectionItem.Types.BARE,
-                collection_item=CollectionItem(
-                    data={
-                        "promise_category": ArtifactCategory.TEST,
-                        "source_package_name": "python3",
-                    }
-                ),
-            )
-        ]
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            r"^testing: unexpected artifact category: 'debusine:test'. "
-            r"Valid categories: \['debian:binary-package'\]$",
-        ):
-            Workflow.get_source_package_names(
-                lookup_result,
-                configuration_key="testing",
-                artifact_expected_categories=(ArtifactCategory.BINARY_PACKAGE,),
-            )
-
-
-class WorkflowInstanceTests(TestCase):
-    """Test instances of the Workflow class."""
-
-    def setUp(self) -> None:
-        """Create a NoopWorkflow for testing."""
-        super().setUp()
-        w = self.playground.create_workflow(task_name="noop")
-        self.workflow = NoopWorkflow(w)
-
-    def test_locate_debian_source_package_lookup_srcpkg(self) -> None:
-        """Test locate_debian_source_package_lookup() with a source package."""
-        source_artifact = self.playground.create_source_artifact()
-        self.assertEqual(
-            self.workflow.locate_debian_source_package_lookup(
-                "source_artifact", source_artifact.id
-            ),
-            source_artifact.id,
-        )
-
-    def test_locate_debian_source_package_lookup_upload(self) -> None:
-        """Test locate_debian_source_package_lookup() with a debian:upload."""
-        artifacts = self.playground.create_upload_artifacts()
-        self.assertEqual(
-            self.workflow.locate_debian_source_package_lookup(
-                "source_artifact", artifacts.upload.id
-            ),
-            f"{artifacts.source.id}@artifacts",
-        )
+from debusine.test.test_utils import preserve_task_registry
 
 
 class OrchestrateWorkflowTests(TestCase):
@@ -326,7 +54,7 @@ class OrchestrateWorkflowTests(TestCase):
             dynamic: str
 
         class ExampleWorkflow(
-            TestTaskMixin, Workflow[BaseWorkflowData, ExampleDynamicData]
+            TestWorkflow[BaseWorkflowData, ExampleDynamicData]
         ):
             TASK_NAME = "example"
 
@@ -363,7 +91,7 @@ class OrchestrateWorkflowTests(TestCase):
             dynamic: str
 
         class ExampleWorkflow(
-            TestTaskMixin, Workflow[BaseWorkflowData, ExampleDynamicData]
+            TestWorkflow[BaseWorkflowData, ExampleDynamicData]
         ):
             TASK_NAME = "example"
 
@@ -560,7 +288,7 @@ class OrchestrateWorkflowTests(TestCase):
             dynamic: str
 
         class ExampleWorkflow(
-            TestTaskMixin, Workflow[BaseWorkflowData, ExampleDynamicData]
+            TestWorkflow[BaseWorkflowData, ExampleDynamicData]
         ):
             TASK_NAME = "example"
 
