@@ -166,7 +166,7 @@ class MakeSignedSourceWorkflowTests(TestCase):
 
     def test_compute_dynamic_data(self) -> None:
         binary_artifact = (
-            self.playground.create_minimal_binary_packages_artifact(
+            self.playground.create_minimal_binary_package_artifact(
                 srcpkg_name="hello"
             )
         )
@@ -262,10 +262,9 @@ class MakeSignedSourceWorkflowTests(TestCase):
         self.assertEqual(sbuild_workflows.count(), len(architectures))
 
         for arch in architectures:
-            collection_id = root.internal_collection_id
             extract_for_signing = extract_for_signings.get(
                 task_data__input__template_artifact__startswith=(
-                    f"{collection_id}@collections/name:signing-template-{arch}-"
+                    f"internal@collections/name:signing-template-{arch}-"
                 )
             )
             self.assertEqual(
@@ -274,10 +273,10 @@ class MakeSignedSourceWorkflowTests(TestCase):
                     "environment": "debian/match:codename=bookworm",
                     "input": {
                         "binary_artifacts": [
-                            f"{collection_id}@collections/name:build-{arch}"
+                            f"internal@collections/name:build-{arch}"
                         ],
                         "template_artifact": (
-                            f"{collection_id}@collections/"
+                            f"internal@collections/"
                             f"name:signing-template-{arch}-hello-{arch}-signed"
                         ),
                     },
@@ -390,7 +389,7 @@ class MakeSignedSourceWorkflowTests(TestCase):
                 {
                     "environment": "debian/match:codename=bookworm",
                     "template": (
-                        f"{collection_id}@collections/"
+                        f"internal@collections/"
                         f"name:signing-template-{arch}-hello-{arch}-signed"
                     ),
                 },
@@ -440,7 +439,10 @@ class MakeSignedSourceWorkflowTests(TestCase):
                     "input": {
                         "source_artifact": (
                             f"internal@collections/name:signed-source-{arch}"
-                        )
+                        ),
+                        "extra_binary_artifacts": [
+                            f"internal@collections/name:build-{arch}"
+                        ],
                     },
                     "target_distribution": "debian:bookworm",
                 },
@@ -558,27 +560,25 @@ class MakeSignedSourceWorkflowTests(TestCase):
         with context.disable_permission_checks():
             architectures = ["amd64"]
 
-            binary_artifact_1 = (
-                self.playground.create_minimal_binary_packages_artifact(
-                    "hello", "1.0.0", "1.0.0", "amd64"
-                )
-            )
-            binary_artifact_2 = (
+            binary_artifact = (
                 self.playground.create_minimal_binary_package_artifact(
-                    "coreutils", "9.1", "9.1-1", "amd64"
+                    "hello", "1.0.0", "hello", "1.0.0", "amd64"
                 )
             )
+            upload_artifact = self.playground.create_upload_artifacts(
+                src_name="coreutils", version="9.1"
+            ).upload
             signing_artifact = self.create_signing_template_artifact("amd64")
 
-            self.playground.create_minimal_binary_packages_artifact(
-                "hello2", "1.0.0", "1.0.0", "i386"
+            self.playground.create_minimal_binary_package_artifact(
+                "hello2", "1.0.0", "hello2", "1.0.0", "i386"
             )
             self.create_signing_template_artifact("i386")
 
         root = self.orchestrate(
             task_data=MakeSignedSourceWorkflowData(
                 binary_artifacts=LookupMultiple.parse_obj(
-                    [binary_artifact_1.id, binary_artifact_2.id]
+                    [binary_artifact.id, upload_artifact.id]
                 ),
                 signing_template_artifacts=LookupMultiple.parse_obj(
                     [signing_artifact.id]
