@@ -247,7 +247,6 @@ class QAWorkflowTests(TestCase):
         )
 
         assert workflow.parent is not None
-        collection_id = workflow.parent.internal_collection_id
         self.assertEqual(autopkgtest.status, WorkRequest.Statuses.RUNNING)
         self.assertEqual(
             autopkgtest.task_data,
@@ -255,9 +254,7 @@ class QAWorkflowTests(TestCase):
                 "arch_all_host_architecture": "amd64",
                 "architectures": ["amd64"],
                 "backend": BackendType.AUTO,
-                "binary_artifacts": [
-                    f"{collection_id}@collections/name:build-amd64"
-                ],
+                "binary_artifacts": ["internal@collections/name:build-amd64"],
                 "codename": "bookworm",
                 "source_artifact": source_artifact.id,
                 "vendor": "debian",
@@ -313,9 +310,7 @@ class QAWorkflowTests(TestCase):
             {
                 "architectures": ["amd64"],
                 "backend": BackendType.AUTO,
-                "binary_artifacts": [
-                    f"{collection_id}@collections/name:build-amd64"
-                ],
+                "binary_artifacts": ["internal@collections/name:build-amd64"],
                 "codename": "bookworm",
                 "fail_on_severity": "none",
                 "source_artifact": source_artifact.id,
@@ -351,8 +346,6 @@ class QAWorkflowTests(TestCase):
             task_name="piuparts", task_type=TaskTypes.WORKFLOW
         )
 
-        collection_id = workflow.parent.internal_collection_id
-
         self.assertEqual(piuparts.status, WorkRequest.Statuses.RUNNING)
         self.assertEqual(
             piuparts.task_data,
@@ -360,9 +353,7 @@ class QAWorkflowTests(TestCase):
                 "arch_all_host_architecture": "amd64",
                 "architectures": ["amd64"],
                 "backend": BackendType.AUTO,
-                "binary_artifacts": [
-                    f"{collection_id}@collections/name:build-amd64"
-                ],
+                "binary_artifacts": ["internal@collections/name:build-amd64"],
                 "codename": "bookworm",
                 "vendor": "debian",
                 "extra_repositories": [
@@ -559,15 +550,12 @@ class QAWorkflowTests(TestCase):
         )
 
         assert workflow.parent is not None
-        collection_id = workflow.parent.internal_collection_id
         self.assertEqual(rdep_autopkgtest.status, WorkRequest.Statuses.RUNNING)
         self.assertEqual(
             rdep_autopkgtest.task_data,
             {
                 "source_artifact": source_artifact.id,
-                "binary_artifacts": [
-                    f"{collection_id}@collections/name:build-amd64"
-                ],
+                "binary_artifacts": ["internal@collections/name:build-amd64"],
                 "suite_collection": f"sid@{CollectionCategory.SUITE}",
                 "vendor": "debian",
                 "codename": "bookworm",
