@@ -654,7 +654,6 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
                     ).populate()
 
             root = self.playground.create_workflow(task_name="examplepipeline")
-            assert (internal_collection := root.internal_collection) is not None
 
             root.mark_running()
             orchestrate_workflow(root)
@@ -721,7 +720,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
                                 f"{sid.id}@collections/name:{source}_all"
                             ],
                             "context_artifacts": [
-                                f"{internal_collection.id}@collections/"
+                                f"internal@collections/"
                                 f"name:build-{architecture}"
                                 for architecture in ("all", "amd64")
                             ],
