@@ -23,10 +23,16 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
     DebianBinaryPackage,
+    DebianBinaryPackages,
+    DebianUpload,
     EmptyArtifactData,
 )
 from debusine.artifacts.playground import ArtifactPlayground
-from debusine.client.models import ArtifactResponse, RemoteArtifact
+from debusine.client.models import (
+    ArtifactResponse,
+    RelationType,
+    RemoteArtifact,
+)
 from debusine.tasks import ExtractForSigning, TaskConfigError
 from debusine.tasks.extract_for_signing import ExtractError
 from debusine.tasks.models import (
@@ -174,6 +180,97 @@ class ExtractForSigningTests(
             ),
         )
 
+    def test_compute_dynamic_data_from_upload(self) -> None:
+        """compute_dynamic_data follows relations from upload artifacts."""
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:format=tarball:"
+                    "backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.template_artifact
+                (2, None): ArtifactInfo(
+                    id=2,
+                    category=ArtifactCategory.BINARY_PACKAGE,
+                    data=DebianBinaryPackage(
+                        srcpkg_name="template",
+                        srcpkg_version="1.0",
+                        deb_fields={"Package": "hello"},
+                        deb_control_files=[],
+                    ),
+                ),
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (LookupMultiple.parse_obj([3, 4]), None): [
+                    ArtifactInfo(
+                        id=3,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "all",
+                                "Files": [{"name": "unsigned-doc_1.0_all.deb"}],
+                            },
+                        ),
+                    ),
+                    ArtifactInfo(
+                        id=4,
+                        category=ArtifactCategory.UPLOAD,
+                        data=DebianUpload(
+                            type="dpkg",
+                            changes_fields={
+                                "Architecture": "amd64",
+                                "Files": [
+                                    {"name": "unsigned-1_1.0_amd64.deb"},
+                                    {"name": "unsigned-2_1.0_amd64.deb"},
+                                ],
+                            },
+                        ),
+                    ),
+                ]
+            },
+            relations={
+                (
+                    (3, 4),
+                    ArtifactCategory.BINARY_PACKAGE,
+                    RelationType.EXTENDS,
+                ): [
+                    ArtifactInfo(
+                        id=artifact_id,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=DebianBinaryPackage(
+                            srcpkg_name="unsigned",
+                            srcpkg_version="1.0",
+                            deb_fields={"Package": binary_package_name},
+                            deb_control_files=[],
+                        ),
+                    )
+                    for artifact_id, binary_package_name in (
+                        (5, "unsigned-doc"),
+                        (6, "unsigned-1"),
+                        (7, "unsigned-2"),
+                    )
+                ]
+            },
+        )
+
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_db),
+            ExtractForSigningDynamicData(
+                environment_id=1,
+                input_template_artifact_id=2,
+                input_binary_artifacts_ids=[5, 6, 7],
+                subject="hello",
+            ),
+        )
+
     def test_compute_dynamic_data_raise_task_config_error_template(
         self,
     ) -> None:
@@ -231,6 +328,69 @@ class ExtractForSigningTests(
         ):
             self.task.compute_dynamic_data(task_db)
 
+    def test_compute_dynamic_data_raise_task_config_error_binary(
+        self,
+    ) -> None:
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:format=tarball:"
+                    "backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.template_artifact
+                (2, None): ArtifactInfo(
+                    id=2,
+                    category=ArtifactCategory.BINARY_PACKAGE,
+                    data=DebianBinaryPackage(
+                        srcpkg_name="template",
+                        srcpkg_version="1.0",
+                        deb_fields={"Package": "hello"},
+                        deb_control_files=[],
+                    ),
+                ),
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (LookupMultiple.parse_obj([3, 4]), None): [
+                    ArtifactInfo(
+                        id=3,
+                        category=ArtifactCategory.BINARY_PACKAGES,
+                        data=DebianBinaryPackages(
+                            srcpkg_name="unsigned",
+                            srcpkg_version="1.0",
+                            version="1.0",
+                            architecture="amd64",
+                            packages=["unsigned-1"],
+                        ),
+                    ),
+                    ArtifactInfo(
+                        id=4,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=DebianBinaryPackage(
+                            srcpkg_name="unsigned",
+                            srcpkg_version="1.0",
+                            deb_fields={"Package": "unsigned-2"},
+                            deb_control_files=[],
+                        ),
+                    ),
+                ]
+            },
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r"^input.binary_artifacts\[0\]: unexpected artifact category: "
+            r"'debian:binary-packages'. "
+            r"Valid categories: \['debian:binary-package', 'debian:upload'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db)
+
     def test_fetch_input_template_wrong_category(self) -> None:
         """fetch_input checks the category of the template artifact."""
         self.task.work_request_id = 147
@@ -301,35 +461,6 @@ class ExtractForSigningTests(
             files={f"{name}_{version}_amd64.deb": create_file_response()},
         )
 
-    def test_fetch_input_binary_wrong_category(self) -> None:
-        """fetch_input checks the category of binary artifacts."""
-        self.task.work_request_id = 147
-        self.task.dynamic_data = ExtractForSigningDynamicData(
-            environment_id=1,
-            input_template_artifact_id=2,
-            input_binary_artifacts_ids=[3, 4],
-        )
-        debusine_mock = self.mock_debusine()
-        debusine_mock.download_artifact.side_effect = [
-            self._create_binary_package_response(
-                2, "hello-signed-template", "1.0+1"
-            ),
-            create_artifact_response(id=3, category=ArtifactCategory.TEST),
-        ]
-
-        self.assertFalse(
-            self.task.fetch_input(self.create_temporary_directory())
-        )
-
-        assert self.task._debug_log_files_directory is not None
-        self.assertEqual(
-            Path(
-                self.task._debug_log_files_directory.name, "fetch_input.log"
-            ).read_text(),
-            "Expected each of binary_artifacts to be of category "
-            "debian:binary-package; got debusine:test\n",
-        )
-
     def test_fetch_input_gathers_information(self) -> None:
         """fetch_input gathers information from artifacts."""
         self.task.work_request_id = 147
