@@ -55,7 +55,7 @@ from debusine.tasks.tests.helper_mixin import (
     FakeTaskDatabase,
 )
 from debusine.test import TestCase
-from debusine.test.utils import create_system_tarball_data
+from debusine.test.test_utils import create_system_tarball_data
 
 
 class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
