@@ -23,6 +23,7 @@ from django.urls import reverse
 from debusine.db.context import context
 from debusine.db.models import Group, Scope, Token, User
 from debusine.db.models.auth import (
+    GroupAuditLog,
     GroupMembership,
     GroupQuerySet,
     Identity,
@@ -363,6 +364,11 @@ class GroupDetailView(
             **kwargs,
         )
         ctx["can_manage"] = self.object.can_manage(context.require_user())
+        audit_log = self.object.audit_log.order_by(
+            "-created_at"
+        ).select_related("actor")[:6]
+        ctx["audit_log"] = audit_log[:5]
+        ctx["audit_log_more"] = len(audit_log) == 6
         try:
             ctx["role"] = GroupMembership.objects.get(
                 group=self.object, user=context.user
@@ -478,3 +484,57 @@ class MembershipDeleteView(
     def get_success_url(self) -> str:
         """Redirect to the view to see the created artifact."""
         return self.object.group.get_absolute_url()
+
+
+class GroupAuditLogView(ListViewBase[GroupAuditLog], BaseUIView):
+    """View for showing the full group audit log."""
+
+    model = GroupAuditLog
+    template_name = "web/group_audit_log-list.html"
+    context_object_name = "audit_log"
+
+    @cached_property
+    def group(self) -> Group:
+        """Return the group for the audit log."""
+        return cast(
+            Group,
+            get_object_or_404(
+                cast(
+                    GroupQuerySet[Any],
+                    Group.objects.can_display_audit_log(
+                        context.require_user()
+                    ).filter(scope=context.require_scope()),
+                ),
+                name=self.kwargs["group"],
+            ),
+        )
+
+    def get_queryset(self) -> QuerySet[GroupAuditLog]:
+        """All audit log entries for the current group."""
+        return self.group.audit_log.select_related("actor")
+
+    def get_title(self) -> str:
+        """Return the page title."""
+        return f"Audit log for {self.group}"
+
+    def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
+        """Add bits needed by the template."""
+        ctx = super().get_context_data(**kwargs)
+        pagination = Pagination(
+            self.request, self.get_queryset(), default_ordering="-created_at"
+        )
+        pagination.add_column(
+            "date",
+            "Date",
+            Ordering("created_at"),
+        )
+        pagination.add_column(
+            "actor",
+            "User",
+            Ordering(
+                asc=["actor__username", "-created_at"],
+                desc=["-actor__username", "-created_at"],
+            ),
+        )
+        ctx["pagination"] = pagination
+        return ctx
