@@ -10,6 +10,7 @@
 """Tests for the auth views."""
 import re
 from datetime import timedelta
+from textwrap import dedent
 from typing import Any, ClassVar
 from urllib.parse import urlencode
 
@@ -21,8 +22,9 @@ from django.test import override_settings
 from django.urls import reverse
 from rest_framework import status
 
-from debusine.db.models import Group, Token, User
-from debusine.db.models.auth import GroupMembership, Identity
+from debusine.db.context import context
+from debusine.db.models import Group, Token, User, auth
+from debusine.db.models.auth import GroupAuditLog, GroupMembership, Identity
 from debusine.db.playground import scenarios
 from debusine.server.signon.providers import Provider
 from debusine.test.django import (
@@ -255,15 +257,17 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
         fqdn = settings.DEBUSINE_FQDN
         self.assertTextContentEqual(
             sample_cfg,
-            f"""
-    [General]
-    default-server = {fqdn}
+            dedent(
+                f"""
+                [General]
+                default-server = {fqdn}
 
-    [server:{fqdn}]
-    api-url = https://{fqdn}/api
-    scope = {self.scenario.scope}
-    token = generated-token-goes-here
-    """,
+                [server:{fqdn}]
+                api-url = https://{fqdn}/api
+                scope = {self.scenario.scope}
+                token = generated-token-goes-here
+                """
+            ),
         )
         self.assertFalse(tree.xpath("ul[@id='scope-list']"))
 
@@ -282,15 +286,17 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
         fqdn = settings.DEBUSINE_FQDN
         self.assertTextContentEqual(
             sample_cfg,
-            f"""
-    [General]
-    default-server = {fqdn}
+            dedent(
+                f"""
+                [General]
+                default-server = {fqdn}
 
-    [server:{fqdn}]
-    api-url = https://{fqdn}/api
-    scope = one-of-the-scopes-listed-below
-    token = generated-token-goes-here
-    """,
+                [server:{fqdn}]
+                api-url = https://{fqdn}/api
+                scope = one-of-the-scopes-listed-below
+                token = generated-token-goes-here
+                """
+            ),
         )
         ul = self.assertHasElement(tree, "//ul[@id='scope-list']")
         self.assertTextContentEqual(ul.li[0], "debusine")
@@ -1021,8 +1027,9 @@ class GroupDetailViewTests(ViewTestMixin, TestCase):
 
     def test_get_ephemeral(self) -> None:
         """View the page as a group member."""
-        self.group.ephemeral = True
-        self.group.save()
+        with context.disable_permission_checks():
+            self.group.ephemeral = True
+            self.group.save()
         response = self.assertGetSucceeds(self.group)
         tree = self.assertHTMLValid(response)
         self.assertEphemeral(tree, True)
@@ -1547,6 +1554,97 @@ class MembershipDeleteViewTests(ViewTestMixin, TestCase):
         self.assertIsMember(self.other_user)
 
 
+class GroupAuditLogViewTests(ViewTestMixin, TestCase):
+    """Tests for :py:class:`GroupAuditLogView`."""
+
+    scenario = scenarios.DefaultContext()
+    group: ClassVar[Group]
+    other_user: ClassVar[User]
+
+    @classmethod
+    def setUpTestData(cls) -> None:
+        """Initialize class data."""
+        super().setUpTestData()
+        cls.other_user = cls.playground.create_user("other")
+        cls.group = cls.playground.create_group("testgroup")
+        changes = auth.GroupAuditLogMemberRemoved(user="…")
+        for n in range(0, 6, 2):
+            changes.user = f"{n}"
+            GroupAuditLog.objects.create(
+                group=cls.group, actor=cls.scenario.user, changes=changes.dict()
+            )
+            changes.user = f"{n + 1}"
+            GroupAuditLog.objects.create(
+                group=cls.group, actor=cls.other_user, changes=changes.dict()
+            )
+
+    def assertGetSucceeds(self) -> TestResponseType:
+        """GET the page and ensure the request succeeds."""
+        self.client.force_login(self.scenario.user)
+        response = self.client.get(
+            reverse("groups:audit-log", kwargs={"group": self.group.name})
+        )
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        return response
+
+    def assertHasRows(
+        self, tree: lxml.objectify.ObjectifiedElement
+    ) -> list[int]:
+        """Ensure the table has rows and return the list of messages."""
+        res: list[int] = []
+        table = self.assertHasElement(tree, "//table[@id='audit-log']")
+        for tr in table.tbody.tr:
+            message = self.get_node_text_normalized(tr.td[2])
+            res.append(int(message.removeprefix("removed ")))
+        self.assertTrue(res)
+        return res
+
+    def test_permissions(self) -> None:
+        """Test that the right permissions are enforced."""
+        url = reverse("groups:audit-log", kwargs={"group": self.group.name})
+        self.client.force_login(self.scenario.user)
+
+        self.assertEnforcesPermission(
+            self.group.can_display,
+            url,
+            "get_context_data",
+            method="get",
+            error_code=status.HTTP_404_NOT_FOUND,
+        )
+
+    def test_get_as_anonymous(self) -> None:
+        """Test that the right permissions are enforced."""
+        response = self.client.get(
+            reverse("groups:audit-log", kwargs={"group": self.group.name})
+        )
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
+
+    def test_get_as_nonmember(self) -> None:
+        """View the page as a group member."""
+        response = self.assertGetSucceeds()
+        tree = self.assertHTMLValid(response)
+        rows = self.assertHasRows(tree)
+        self.assertEqual(rows, [5, 4, 3, 2, 1, 0])
+
+    def test_get_as_member(self) -> None:
+        """View the page as a group member."""
+        self.playground.add_user(self.group, self.scenario.user)
+        response = self.assertGetSucceeds()
+        tree = self.assertHTMLValid(response)
+        rows = self.assertHasRows(tree)
+        self.assertEqual(rows, [5, 4, 3, 2, 1, 0])
+
+    def test_get_as_admin(self) -> None:
+        """View the page as a group member."""
+        self.playground.add_user(
+            self.group, self.scenario.user, Group.Roles.ADMIN
+        )
+        response = self.assertGetSucceeds()
+        tree = self.assertHTMLValid(response)
+        rows = self.assertHasRows(tree)
+        self.assertEqual(rows, [5, 4, 3, 2, 1, 0])
+
+
 def _html_collapse_token(key: str) -> str:
     """Return HTML for a show/hide button for token key."""
     return f'''<button class="btn btn-primary btn-sm" type="button"
