@@ -32,7 +32,6 @@ from debusine.server.workflows.models import WorkRequestManualUnblockAction
 from debusine.tasks import TaskConfigError
 from debusine.tasks.models import TaskTypes
 from debusine.web.forms import WorkRequestForm, WorkRequestUnblockForm
-from debusine.web.utils import ui_prototype
 from debusine.web.views import sidebar, ui_shortcuts
 from debusine.web.views.base import (
     CreateViewBase,
@@ -44,7 +43,8 @@ from debusine.web.views.base import (
 )
 from debusine.web.views.base_rightbar import RightbarUIView
 from debusine.web.views.http_errors import HttpError400, catch_http_errors
-from debusine.web.views.utils import PaginationMixin, format_yaml
+from debusine.web.views.view_utils import PaginationMixin, format_yaml
+from debusine.web.web_utils import ui_prototype
 
 
 class WorkRequestObjectMixin(WorkspaceView):
