@@ -18,7 +18,7 @@ from debusine.db.models.work_requests import WorkRequestQuerySet
 from debusine.tasks.models import TaskTypes
 from debusine.web.forms import WorkflowFilterForm
 from debusine.web.views.base import ListViewBase, WorkspaceView
-from debusine.web.views.utils import PaginationMixin
+from debusine.web.views.view_utils import PaginationMixin
 
 
 class WorkflowsListView(
